#!/bin/bash
version="v0.7 Dec15"  # by Joerg Schulenburg
#
#  generate new daten.def (spin-1) from old daten.def (spin-1/2)  
#  use:     ./defspin1.sh -s {2...n} < old.def > new.def
#           ./m_1d 5 | ../utils/defspin1.sh -s 2 > daten.def
#  options:
#    -s n   -  n ist the spin length in units of 1/2
#    -h     -  this help
#    -a     -  alternative site order
#
# v0.7 Dec15 default file is stdin instead of daten.def 
# v0.6 May12 alternative order of sites -a (better to debug, faster code?)
# v0.5 Sep10  output positions as float (old=int), support new def-format
# v0.4 Oct07  new daten.def adaptions, tail 5.97 options
# v0.3 Feb04  # by Joerg Schulenburg
# bugs: no support for quad-terms
#
LANG=C    # printf %f of awk is language dependent, we want the x.y-format
ns=0    # this is the default s=1/2
order=0 # order of copied sites
while test "${1:0:1}" = "-" && test "$1" != "-"; do # only bash: first char of $1
  if   test "$1" = "-v"; then shift 
  elif test "$1" = "-s"; then ns=$2; shift 2
  elif test "$1" = "-a"; then order=1; shift
  else echo "unknown option: $1"; tail -n +2 $0 | head -10 1>&2 ; exit
  fi
done
# for 0 arguments show head lines of this script (=manual)
if test $ns -lt 1; then tail -n +2 $0 | head -11 1>&2 ; exit; fi
# ns=2 doubles spin length, ns=3 triples spin length
# 1. Argument ist Dateiname des def-files oder "-" for stdin
#
#if test -z "$1"; then file=daten.def
#                 else file=$1
#fi
if test -n "$1"; then file="$1";fi

if test "$file" != "-" && test ! -r $file; then echo "file $file not found";exit; fi
echo "# this file was automaticly generated by defspin1.sh ${version}"
#
#
if [ $order == 1 ]; then  # 1a 1b 2a 2b 3a 3b ... alternative order
awk -v ns=$ns '\
  BEGIN {\
     printf "# enlarge S by factor %d (option -a, alternative order)\n",ns;
     count=0;nr=0;b1x=1;b1y=0;b2x=0;b2y=-1;e1x=0;e1y=0;e2x=0;e2y=0;
  }
  !/^\#/ { count++; nr=count; }      # count line number
  /^\#/  { nr=0; printf "%s\n",$0 }  # comment
  (nr == 1) { vn=$1;vnw=$2;\
   printf "# ns=%d NN=ns*%d Nw=ns*ns*%d\n%d %d 0\n",ns,vn,vnw,ns*vn,ns*ns*vnw }
  # read in sites
  (nr > 1) && (nr < vn+2) {\
    i=nr-2; # 0..vn-1, old i=$1;
    ax[i]=$2; ay[i]=$3; az[i]=$4; aa[i]=""
    for (j=5;j<=NF;j++) aa[i]=aa[i]" "$j;
  }
  # output 2n sites
  (nr == vn+1){\
    for(i=0;i<vn;i++)for(j=0;j<ns;j++)\
    printf "%3d % 4.2f % 4.2f % 4.2f %s\n", i*ns+j,ax[i],ay[i],az[i],aa[i];
  }
  # read bonds
  (nr > vn+1) {\
#   if (match($0,"+"))
    i=nr-vn-2; # 0..vn-1
    b1[i]=$2; b2[i]=$3; bb[i]="";
    for (j=4;j<=NF;j++) bb[i]=bb[i]" "$j;
  }
  # output ns*ns*nw bonds
  (nr == vn+1+vnw){\
    for(i=0;i<vnw;i++)for(j1=0;j1<ns;j1++)for(j2=0;j2<ns;j2++)\
    printf "%3d %2d %2d %s\n", i*ns*ns+(j1*ns+j2),j1+b1[i]*ns,j2+b2[i]*ns,bb[i];
  }
  ' $file
else                     # 1a 2a 3a 4a ... 1b 2b 3b 4b ... old order
awk -v ns=$ns '\
  BEGIN {\
     printf "# enlarge S by factor %d\n",ns;
     count=0;nr=0;b1x=1;b1y=0;b2x=0;b2y=-1;e1x=0;e1y=0;e2x=0;e2y=0;
  }
  !/^\#/ { count++; nr=count; }      # count line number
  /^\#/  { nr=0; printf "%s\n",$0 }  # comment
  (nr == 1) { vn=$1;vnw=$2;\
   printf "# ns=%d NN=ns*%d Nw=ns*ns*%d\n%d %d 0\n",ns,vn,vnw,ns*vn,ns*ns*vnw }
  # read in sites
  (nr > 1) && (nr < vn+2) {\
    i=nr-2; # 0..vn-1, old i=$1;
    ax[i]=$2; ay[i]=$3; az[i]=$4; aa[i]=""
    for (j=5;j<=NF;j++) aa[i]=aa[i]" "$j;
  }
  # output 2n sites
  (nr == vn+1){\
    for(j=0;j<ns;j++)for(i=0;i<vn;i++)\
    printf "%3d % 4.2f % 4.2f % 4.2f %s\n", i+j*vn,ax[i],ay[i],az[i],aa[i];
  }
  # read bonds
  (nr > vn+1) {\
#   if (match($0,"+"))
    i=nr-vn-2; # 0..vn-1
    b1[i]=$2; b2[i]=$3; bb[i]="";
    for (j=4;j<=NF;j++) bb[i]=bb[i]" "$j;
  }
  # output ns*ns*nw bonds
  (nr == vn+1+vnw){\
    for(j1=0;j1<ns;j1++)for(j2=0;j2<ns;j2++)for(i=0;i<vnw;i++)\
    printf "%3d %2d %2d %s\n", i+(j1*ns+j2)*vnw,j1*vn+b1[i],j2*vn+b2[i],bb[i];
  }
  ' $file
fi
exit

